from typing import Dict


class AnnotationRelation:

  @staticmethod
  def create_api_relation(dict: Dict):
    """
    Maps an sdk relation dict to an api annotation relation dict, for use in calls to the api.

    AnnotationRelation parameters:
      annotations: [Union[str, int], Union[str, int]]
      type:        str
      schema:      str
      value:       str
      reviewed     bool (Optional)
    """
    relation = {
      'predecessorId': str(dict['annotations'][0]),
      'successorId': str(dict['annotations'][1])
    }

    if ('type' in dict): relation['typeIdentifier'] = dict['client_id']
    if ('schema' in dict): relation['schemaIdentifier'] = dict['schema']
    if ('value' in dict): relation['value'] = dict['value']
    if ('reviewed' in dict): relation['isReviewed'] = dict['reviewed']

    return relation


from annolab import endpoints
from annolab.project import Project
from annolab.api_helper import ApiHelper

class AnnoLab:

  def __init__(
    self,
    api_key = None,
    api_url = 'https://api.annolab.ai',
  ):
    self.__api = ApiHelper(api_key=api_key, api_url=api_url)

  @property
  def api_key_info(self):
    return self.__api.api_key_info


  @property
  def default_owner(self):
    """
      Returns the default group to use for the api key.
      The default group is the group representing the single user.
    """
    return self.__api.default_owner


  def find_project(self, name: str, owner_name: str = None):
    """
      Find a project by name and (optionally) group name.
      If group name is not passed, the user's default group is used.
    """
    owner_name = owner_name or self.default_owner['groupName']

    res = self.__api.get_request(
      endpoints.Project.get_group_project(owner_name, name)
    )

    return Project.create_from_response_json(res.json(), self.__api)


  def create_project(self, name: str, owner_name: str = None):
    """
      Create a project.
      If group name is not passed, the user's default group is used.
    """
    owner_name = owner_name or self.default_owner['groupName']

    res = self.__api.post_request(
      endpoints.Project.post_create(),
      {
        'name': name,
        'groupName': owner_name
      }
    )

    return Project.create_from_response_json(res.json(), self.__api)



class Project:

  @staticmethod
  def post_create():
    return 'v1/project/create'


  @staticmethod
  def get_group_project(groupName: str, projectName: str):
    return f'v1/project/{groupName}/{projectName}'


  @staticmethod
  def get_using_id(id: int):
    return f'v1/project/{id}'


class Export:

  @staticmethod
  def post_export_project():
    return 'v1/export/project'


class Directory:

  @staticmethod
  def post_create():
    return 'v1/directory/create'


class Annotation:

  @staticmethod
  def post_create():
    return 'v1/annotation/create'

  @staticmethod
  def post_bulk_create():
    return 'v1/annotation/bulk-create'


class AnnotationType:

  @staticmethod
  def post_create():
    return 'v1/annotation-type/create'


class AnnotationSchema:

  @staticmethod
  def post_create():
    return 'v1/schema/create'


class AnnotationRelation:

  @staticmethod
  def post_create():
    return 'v1/relation/create'


  @staticmethod
  def post_bulk_create():
    return 'v1/relation/bulk-create'


class AnnotationLayer:

  @staticmethod
  def create():
    return 'v1/layer/create'


class Source:

  @staticmethod
  def get_source_by_id(source_ref_id: int):
    return f'v1/source/{source_ref_id}'


  @staticmethod
  def get_source_by_path(owner_name: str, project_name: str, directory_name: str, source_ref_name: str):
    return f'v1/source/{owner_name}/{project_name}/{directory_name}/{source_ref_name}'


  @staticmethod
  def post_create_text():
    return 'v1/source/create-text'


  @staticmethod
  def post_initialize_pdf():
    return 'v1/source/init-pdf'


  @staticmethod
  def post_create_pdf():
    return 'v1/source/create-pdf'


  @staticmethod
  def post_annotations(owner_name: str, project_name: str, directory_name: str, source_ref_name: str):
    return f'v1/source/{owner_name}/{project_name}/{directory_name}/{source_ref_name}/annotations'


  @staticmethod
  def delete_using_name(owner_name: str, project_name: str, directory_name: str, source_ref_name: str):
    return f'v1/source/{owner_name}/{project_name}/{directory_name}/{source_ref_name}'


  @staticmethod
  def delete_using_id(id: int):
    return f'v1/source/{id}'


class ApiKey:
  @staticmethod
  def get_api_key_info():
    return 'v1/api-key/info'


#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
  name = 'annolab',
  version='0.1.3',
  packages = find_packages(),
  author='AnnoLab',
  maintainer='Luke Simkins; Grant DeLozier',
  author_email='luke@annolab.ai',
  url='https://github.com/lsimkins/annolab-sdk',
  license='Apache',
  description='Official SDK for the AnnoLab Platform',
  long_description=open('README.rst').read(),
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
  ],
)


import io
from os import path
from typing import Any, Dict, List, Union
import requests

from annolab import endpoints
from annolab.api_helper import ApiHelper
from annolab.annotation import Annotation
from annolab.annotation_relation import AnnotationRelation

class Project:

  def __init__(
    self,
    name: str,
    id: int,
    owner_name: str,
    owner_id: int,
    default_dir: str,
    api_helper: ApiHelper
  ):
    self.name = name
    self.id = id
    self.owner_name = owner_name
    self.owner_id = owner_id
    self.default_dir = default_dir
    self.__api = api_helper


  @property
  def project_path(self):
    return f'{self.owner_name}/{self.name}'


  def find_source(self, name: str, directory: str = None):
    """
      Search for a source within a project by name and (optionally) directory.
      If directory is not provided, the default directory is used (typically "Uploads").
    """
    res = self.__api.get_request(
      endpoints.Source.get_source_by_path(
        owner_name=self.__api.default_owner['groupName'],
        project_name=self.name,
        directory_name=directory or self.default_dir,
        source_ref_name=name
      )
    )

    return res.json()


  def create_text_source(self, name: str, text: str, directory: str = None):
    """
      Creates a text source.
      If directory is not provided, the default directory is used (typically "Uploads").
    """
    body = {
      'projectIdentifier': self.id or self.name,
      'groupName': self.owner_name,
      'sourceName': name,
      'text': text
    }

    if (directory is not None):
      body['directoryIdentifier'] = directory

    res = self.__api.post_request(
      endpoints.Source.post_create_text(),
      body
    )

    return res.json()


  def create_pdf_source(self, file: Union[str, io.IOBase, bytes], name: str = None, directory: str = None):
    """
      Creates a pdf source from a local file, bytes, or filelike object.
      If directory is not provided, the default directory is used (typically "Uploads").
    """
    is_io_or_bytes = isinstance(file, io.IOBase) or isinstance(file, bytes)
    if (is_io_or_bytes and name is None):
      raise Exception('You must provide a name when passing a filelike object for pdf source creation')

    name = name or path.basename(file)

    init_res = self.__api.post_request(
      endpoints.Source.post_initialize_pdf(),
      {
        'projectIdentifier': self.id or self.name,
        'groupName': self.owner_name,
        'directoryIdentifier': directory or self.default_dir,
        'sourceName': name,
      })

    upload_url = init_res.json()['uploadUrl']

    pdf_file = file if is_io_or_bytes else open(file, 'r+b')
    self.__api.put_request(upload_url, data = pdf_file, headers={'Content-Type': 'application/pdf'})

    if (not isinstance(file, bytes)):
      pdf_file.close()

    create_res = self.__api.post_request(
      endpoints.Source.post_create_pdf(),
      {
        'projectIdentifier': self.id or self.name,
        'groupName': self.owner_name,
        'directoryIdentifier': directory or self.default_dir,
        'sourceIdentifier': name,
      })

    return create_res.json()


  def create_pdf_source_from_web(self, url: str, name: str = None, directory: str = None):
    """
      Creates a pdf source from a web url.
      If directory is not provided, the default directory is used (typically "Uploads").
    """
    name = name or path.basename(url)
    res = requests.get(url)

    if (res.status_code >= 300):
      res.raise_for_status()

    return self.create_pdf_source(res.content, name, directory)


  def create_annotations(
    self,
    source_name: str,
    annotations: List[Any],
    relations: List[Any] = [],
    dedup: bool = True,
    directory: str = None):
    """
      Create annotations against a source.

      Annotation parameters:
        type:      str  (Required)
        client_id: str  (Optional, Required if passing relations)
        schema:    str  (Optional)
        value:     str  (Optional)
        offsets:   [int, int] (Optional)
        bbox:      [int, int, int, int] (Optional)
        layer:     str  (Optional)
        page:      int  (Optional)
        reviewed   bool (Optional)
    """
    directory = directory or self.default_dir

    res = self.__api.post_request(
      endpoints.Source.post_annotations(self.owner_name, self.name, directory, source_name),
      {
        'annotations': list(map(Annotation.create_api_annotation, annotations)),
        'relations': list(map(AnnotationRelation.create_api_relation, relations)),
        'preventDuplication': dedup
      })

    return res.json()


  def export(
    self,
    filepath: str,
    source_ids: List[int] = None,
    layers: List[str] = None,
    include_schemas: bool = False,
    include_sources: bool = False
  ):
    body = {
      'projectIdentifier': self.name,
      'groupName' : self.owner_name,
      'includeSchemas': include_schemas,
      'includeSources': include_sources
    }

    if (source_ids is not None): body['sourceIds'] = source_ids
    if (layers is not None): body['annotationLayerNames'] = layers

    res = self.__api.post_request(
      endpoints.Export.post_export_project(),
      body
    )

    with open(filepath, 'wb') as f:
      f.write(res.content)


  @staticmethod
  def create_from_response_json(resp_json: Dict, api_helper: ApiHelper):
    return Project(
      resp_json['name'],
      resp_json['id'],
      resp_json['groupName'],
      resp_json['groupId'],
      resp_json['defaultDirectory'],
      api_helper=api_helper)



from typing import Dict

class Annotation:

  @staticmethod
  def create_api_annotation(dict: Dict):
    """
    Maps an sdk annotation dict to an api annotation dict, for use in calls to the api.

    Annotation parameters:
      type:      str  (Required)
      client_id: str  (Optional, Required if passing relations)
      schema:    str  (Optional)
      value:     str  (Optional)
      offsets:   [int, int] (Optional)
      bbox:      [int, int, int, int] (Optional)
      layer:     str  (Optional)
      page:      int  (Optional)
      reviewed   bool (Optional)
    """
    annotation = { 'annoTypeIdentifier': dict['type'] }

    if ('client_id' in dict): annotation['clientId'] = str(dict['client_id'])
    if ('offsets' in dict): annotation['offsets'] = dict['offsets']
    if ('schema' in dict): annotation['schemaIdentifier'] = dict['schema']
    if ('value' in dict): annotation['value'] = dict['value']
    if ('bbox' in dict): annotation['bbox'] = dict['bbox']
    if ('layer' in dict): annotation['layerIdentifier'] = dict['layer']
    if ('page' in dict): annotation['pageNumber'] = dict['page']
    if ('reviewed' in dict): annotation['isReviewed'] = dict['reviewed']

    return annotation